﻿"""
Add text to describe what this macro does
"""
#region Reference stuff

import clr
clr.AddReference('IronPython.Wpf')
import wpf
import datetime
import random
import string
import re




from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File

clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys, OpenFileDialog, SaveFileDialog
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Sdk.ForeignCad")
from Trimble.Vce.ForeignCad import TextEntity, MText, Text, Leader, Point
clr.AddReference("Trimble.Sdk.Geometry")
from Trimble.Vce.Geometry.PolySeg import PolySeg
clr.AddReference("Trimble.Sdk.Interfaces")
from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation, IFeatureCodeParent
from Trimble.Vce.Interfaces.Client import CommandGranularity, IProject, IProjectGetter
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Vce.UI.Controls import VceMessageBox, AngularEdit, BrowseFile
clr.AddReference("Trimble.Sdk.DiskIO")
from Trimble.DiskIO import SearchPath
clr.AddReference("Trimble.Sdk.Coordinates")
from Trimble.Vce.Coordinates import CoordinateData, EnabledStatus, ICoordinate, ICoordinateAccess
from Trimble.Vce.Core import GlobalSelection




def Setup(cmdData, macroFileFolder):
    cmdData.Key = "pxyExport"
    cmdData.CommandName = "pxyExport"
    cmdData.Caption = "_pxy export"
    cmdData.UIForm = "pxyExport" # MUST MATCH NAME FROM CLASS DEFINED BELOW !!!

    # if you are going to include a help file, define file and topic id here
    cmdData.HelpFile = "MyMacros.chm"
    cmdData.HelpTopic = "xxxx"
    # Version 5.10 has macro version support
    try:
        cmdData.Version = 1.11
        cmdData.MacroAuthor = "author of macro..."
        cmdData.MacroInfo = r""
    except:
        pass

    # if you want the macro included in the ribbon, fill in the following 5 lines
    cmdData.DefaultTabKey = "Macros"
    cmdData.DefaultTabGroupKey = "MyStuff"
    cmdData.ShortCaption = "PXY export"
    cmdData.ToolTipTittle = "pxyExport"
    cmdData.ToolTipTextFormatted = "pxyExport"
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

# the name of this class must match name from cmdData.UIForm (above)
class pxyExport(StackPanel): # this inherits from the WPF StackPanel control
    #VceMessageBox.Show("tests")
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\pxyExport.xaml") as s:
            wpf.LoadComponent(self, s)

        self.currentProject = currentProject
        self.tType = clr.GetClrType(TextEntity)
        self.lType = clr.GetClrType(IPolyseg)
        self.pType = clr.GetClrType(IPoint)
        self.ptType = clr.GetClrType(IPointLocation)
        self.fcType = clr.GetClrType(IFeatureCodeParent)


        self.pxy = []
        self.sorteradArray = []
        self.lineArray = []
        #Justera så allatid projektmappen som default
        self.folderDir.Text = str(SearchPath.ExportAppDataPath)+"\default.pxy"
        # self.folderDir.Text = str(SearchPath.DefaultProjectDataPath)+"\\default.pxy"

        self.resetLineArray()


    def OnLoad(self, cmd, buttons, event):
        optionMenu = SelectionContextMenuHandler()
        optionMenu.ExcludedCommands = "SelectObservations | SelectPoints | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu


    def CancelClicked(self, sender, e):
        sender.CloseUICommand()


    def OkClicked(self, sender, e):
        self.getSelected()

        #Soring function for array
        def natural_sort(l):
            convert = lambda text: int(text) if text.isdigit() else text.lower()
            alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ]
            return sorted(l, key = alphanum_key)

        # self.sorteradArray = sorted(self.pxy)
        self.sorteradArray = natural_sort(self.pxy)



        if len(self.sorteradArray) == 0:
            self.objs.StatusMessage = "Nothing Selected"
            return
        else:
            #Datum
            now = str(datetime.datetime.now())

            # pxyReverse = self.sorteradArray[::-1]
            try:
                fh = open(self.folderDir.Text, 'w+')
                fh.write("XYZ-COORD-FILE  ,V1.00," + now[0:10] + ",                                        ,\n")
                fh.write("                                        ,                                 ,\n")
                for coord in self.sorteradArray:
                    fh.write(str(coord))
                self.pxy = []
                # pxyReverse = None
                self.sorteradArray = []
                fh.close()
                sender.CloseUICommand()
            except:
                varning = "\n" + str(self.folderDir.Text) + "\n" + "Du har inte behörighet att spara på denhär platsen.\n"+ "Kontakta administratören om du behöver behörighet."
                VceMessageBox.ShowWarning(varning)
                self.pxy = []
                # pxyReverse = None
                self.sorteradArray = []
                return


    def browse(self, sender, e):
        fileDialog = SaveFileDialog()
        fileDialog.Filter = "pxy files (*.pxy)|*.pxy|All files (*.*)|*.*"  ;
        fileDialog.InitialDirectory = str(SearchPath.DefaultProjectDataPath)
        fileDialog.ShowDialog()
        if len(fileDialog.FileNames) == 1:
            self.folderDir.Text = str(fileDialog.FileNames[0])






    def getSelected(self):
        #Selecting alla points
        for o in self.objs.SelectedMembers(self.currentProject):


            if isinstance(o, self.pType):
                if str(o.Position.X) == "nan" and str(o.Position.Y) == "nan":
                    pass
                else:
                    #Adds id to array
                    if isinstance(o, self.pType):
                        PointIdString = str(o.PointID)
                        testAscii = all(ord(char) < 128 for char in PointIdString)
                        if testAscii == False:
                            PointIdString = ''.join([i if ord(i) < 128 else '?' for i in PointIdString])
                        if len(PointIdString) > 12:
                            svarVarning = VceMessageBox.ShowWarningOKCancel("Punkt ID: " + PointIdString + " är för långt. \nByta punktnamn?")
                            if str(svarVarning) == "OK":
                                randomString = self.get_random_string(9)
                                PointIdString = "ID_" + str(randomString)
                            else:
                                VceMessageBox.Show("Exporten avbruten")
                                self.pxy = []
                                break
                        pointIdArray = self.splitToArray(PointIdString)

                        i = 0
                        for letter in pointIdArray:
                            self.lineArray[i] = letter
                            i = i + 1

                    #Adds coordinates
                    if isinstance(o, self.ptType):
                        xCoordinate3decimals = round(o.Position.X, 3)
                        xCoordinate3dec3zero = format(xCoordinate3decimals, '.3f')
                        yCoordinate3decimals = round(o.Position.Y, 3)
                        yCoordinate3dec3zero = format(yCoordinate3decimals, '.3f')
                        if str(o.Position.Z) == "nan":
                            zCoordinate3dec3zero = "0.000"
                        else:
                            zCoordinate3decimals = round(o.Position.Z, 3)
                            zCoordinate3dec3zero = format(zCoordinate3decimals, '.3f')

                        xArray = self.splitToArray(str(xCoordinate3dec3zero))
                        xLength = len(xArray)
                        xArrayReverse = xArray[::-1]
                        yArray = self.splitToArray(str(yCoordinate3dec3zero))
                        yLength = len(yArray)
                        yArrayReverse = yArray[::-1]
                        zArray = self.splitToArray(str(zCoordinate3dec3zero))
                        zLength = len(zArray)
                        zArrayReverse = zArray[::-1]

                        #OBS! Detta är angivet i XYZ och ej NEE
                        #Viktigt ------------------------------
                        yi = 25
                        for y in yArrayReverse:
                            self.lineArray[yi] = y
                            yi = yi - 1

                        xi = 37
                        for x in xArrayReverse:
                            self.lineArray[xi] = x
                            xi = int(xi) - 1

                        zi = 49
                        for z in zArrayReverse:
                            self.lineArray[zi] = z
                            zi = zi - 1

                    #Adds featurecode to array
                    if isinstance(o, self.fcType):
                        featureCode = str(o.FeatureCode)

                        testAscii = all(ord(char) < 128 for char in featureCode)
                        if testAscii == False:
                            featureCode = ''.join([i if ord(i) < 128 else '?' for i in featureCode])
                        if len(featureCode) > 8:
                            featureCode = featureCode[0:8]
                        fcArray = self.splitToArray(featureCode)

                        i = 51
                        for letter in fcArray:
                            self.lineArray[i] = letter
                            i = i + 1

                    #Adds Description1 to array
                    if hasattr(o, 'Description1'):
                        descr1 = str(o.FullDescription1)
                        testAscii = all(ord(char) < 128 for char in descr1)
                        if testAscii == False:
                            descr1 = ''.join([i if ord(i) < 128 else '?' for i in descr1])
                        descrArray = self.splitToArray(descr1)

                        i = 62
                        for letter in descrArray:
                            self.lineArray[i] = letter
                            i = i + 1
                            if i > 73:
                                break


                    self.lineArray[74] = ",\n"

                    fullString = ("".join(self.lineArray))
                    self.pxy.append(fullString)

                    #Reseting variables
                    self.resetLineArray()
                    fullString = ""
            else:
                pass


    def resetLineArray(self):
        self.lineArray = []
        for i in range(75):
            self.lineArray.append(" ")


    def splitToArray(self, str):
        strArray = list(str)
        return strArray


    def get_random_string(self, length):
        letters = string.ascii_lowercase
        result_str = ''.join(random.choice(letters) for i in range(length))
        return result_str
